@@ -1,7 +1,7 @@
 /*
  * %CopyrightBegin%
  *
- * Copyright Ericsson AB 2018-2023. All Rights Reserved.
+ * Copyright Ericsson AB 2018-2025. All Rights Reserved.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -147,6 +147,40 @@ static SOCKLEN_T sa_local_length(int l, struct sockaddr_un* sa);
 #endif
 
 
+/* The pre string has to be large enough (suggest 64) that
+ * the name fits.
+ * This function creates a mutex with a specific name:
+ *       <pre>[<file descriptor>]
+ * For example: esock.w[10]
+ *
+ * But this is only done if ESOCK_VERBOSE_MTX_NAMES is defined.
+ * Otherwise it will just be the pre string.
+ */
+extern
+ErlNifMutex* esock_mutex_create(const char* pre, char* buf, SOCKET sock)
+{
+#if defined(ESOCK_VERBOSE_MTX_NAMES)
+    /* ESOCK_PRINTF("esock_mutex_create -> create name with" */
+    /*              "\r\n   pre:  %s" */
+    /*              "\r\n   sock: %d" */
+    /*              "\r\n", pre, sock); */
+    sprintf(buf, "%s[" SOCKET_FORMAT_STR "]", pre, sock);
+#else
+    /* ESOCK_PRINTF("esock_mutex_create -> create name with" */
+    /*              "\r\n   pre: %s" */
+    /*              "\r\n", pre); */
+    VOID(sock);
+    sprintf(buf, "%s", pre);
+#endif
+
+    /* ESOCK_PRINTF("esock_mutex_create -> create mtx with name: %s" */
+    /*              "\r\n", buf); */
+
+    return enif_mutex_create(buf);
+}
+
+
+
 /* *** esock_get_uint_from_map ***
  *
  * Simple utility function used to extract a unsigned int value from a map.
@@ -1040,10 +1074,11 @@ void esock_encode_sockaddr_un(ErlNifEnv*          env,
     size_t       n, m;
 
     UDBG( ("SUTIL", "esock_encode_sockaddr_un -> entry with"
-           "\r\n.  addrLen: %d"
+           "\r\n   addrLen: %d"
            "\r\n", addrLen) );
 
     n = sockAddrP->sun_path - (char *)sockAddrP; // offsetof
+
     if (addrLen >= n) {
         n = addrLen - n; // sun_path length
         if (255 < n) {
@@ -1055,6 +1090,7 @@ void esock_encode_sockaddr_un(ErlNifEnv*          env,
             unsigned char *path;
 
             m = esock_strnlen(sockAddrP->sun_path, n);
+
 #ifdef __linux__
             /* Assume that the address is a zero terminated string,
              * except when the first byte is \0 i.e the string length is 0,
@@ -1067,6 +1103,8 @@ void esock_encode_sockaddr_un(ErlNifEnv*          env,
             }
 #endif
 
+            UDBG( ("SUTIL", "esock_encode_sockaddr_un -> m: %d\r\n", m) );
+
             /* And finally build the 'path' attribute */
             path = enif_make_new_binary(env, m, &ePath);
             ESOCK_ASSERT( path != NULL );
@@ -2371,18 +2409,65 @@ ERL_NIF_TERM esock_errno_to_term(ErlNifEnv* env, int err)
         break;
 #endif
 
+#if defined(WSAENOTSOCK)
+    case WSAENOTSOCK:
+        return MKA(env, "enotsock");
+        break;
+#endif
+
 #if defined(ERROR_INVALID_NETNAME)
     case ERROR_INVALID_NETNAME:
         return MKA(env, "invalid_netname");
         break;
 #endif
 
+#if defined(ERROR_NETNAME_DELETED)
+    case ERROR_NETNAME_DELETED:
+        return MKA(env, "netname_deleted");
+        break;
+#endif
+
+#if defined(ERROR_TOO_MANY_CMDS)
+        /* The network command limit has been reached */
+    case ERROR_TOO_MANY_CMDS:
+        return MKA(env, "too_many_cmds");
+        break;
+#endif        
+
+#if defined(ERROR_DUP_NAME)
+        /*  Not connected because a duplicate name exists on the network */
+    case ERROR_DUP_NAME:
+        return MKA(env, "duplicate_name");
+        break;
+#endif        
+
 #if defined(ERROR_MORE_DATA)
+        /*
+         * https://stackoverflow.com/questions/31883438/sockets-using-getqueuedcompletionstatus-and-error-more-data
+         */
     case ERROR_MORE_DATA:
         return MKA(env, "more_data");
         break;
 #endif
 
+#if defined(ERROR_NOT_FOUND)
+    case ERROR_NOT_FOUND:
+        return MKA(env, "not_found");
+        break;
+#endif
+
+#if defined(ERROR_NETWORK_UNREACHABLE)
+    case ERROR_NETWORK_UNREACHABLE:
+        return MKA(env, "network_unreachable");
+        break;
+#endif
+
+#if defined(ERROR_PORT_UNREACHABLE)
+    case ERROR_PORT_UNREACHABLE:
+        return MKA(env, "port_unreachable");
+        break;
+#endif        
+
     default:
         {
             char* str = erl_errno_id(err);
@@ -2504,7 +2589,24 @@ ERL_NIF_TERM esock_make_error_t2r(ErlNifEnv*   env,
                                   ERL_NIF_TERM tag,
                                   ERL_NIF_TERM reason)
 {
-    return MKT2(env, esock_atom_error, MKT2(env, tag, reason));
+    return esock_make_error(env, MKT2(env, tag, reason));
+}
+
+
+
+/* Create an 'error' two (2) tuple in the form:
+ *
+ *          {error, {invalid, {integer_range, I}}}
+ *
+ * The second arg (i) is already in the form of an
+ * ERL_NIF_TERM so all we have to do is create the tuple.
+ */
+extern
+ERL_NIF_TERM esock_make_error_integer_range(ErlNifEnv* env, ERL_NIF_TERM i)
+{
+    ERL_NIF_TERM intRange = MKT2(env, esock_atom_integer_range, i);
+
+    return esock_make_error_invalid(env, intRange);
 }
 
 
@@ -2516,25 +2618,7 @@ ERL_NIF_TERM esock_make_error_t2r(ErlNifEnv*   env,
 extern
 ERL_NIF_TERM esock_make_error_invalid(ErlNifEnv* env, ERL_NIF_TERM what)
 {
-    return MKT2(env,
-                esock_atom_error,
-                MKT2(env, esock_atom_invalid, what));
-}
-
-
-
-/* Create an 'error' two (2) tuple in the form:
- *
- *          {error, {invalid, {integer_range, I}}}
- *
- * The second element (i) is already in the form of an
- * ERL_NIF_TERM so all we have to do is create the tuple.
- */
-extern
-ERL_NIF_TERM esock_make_error_integer_range(ErlNifEnv* env, ERL_NIF_TERM i)
-{
-    return
-        esock_make_invalid(env, MKT2(env, esock_atom_integer_range, i));
+    return esock_make_error_t2r(env, esock_atom_invalid, what);
 }
 
 
@@ -2584,17 +2668,21 @@ size_t esock_strnlen(const char *s, size_t maxlen)
  *
  */
 extern
-void __noreturn
-esock_abort(const char* expr,
-                 const char* func,
-                 const char* file,
-                 int         line)
+void __noreturn esock_abort(const char* expr,
+                            const char* func,
+                            const char* file,
+                            int         line)
 {
+#if 0
     fflush(stdout);
     fprintf(stderr, "%s:%d:%s() Assertion failed: %s\n",
             file, line, func, expr);
     fflush(stderr);
     abort();
+#else
+    erts_exit(ERTS_DUMP_EXIT, "%s:%d:%s() Assertion failed: %s\n",
+              file, line, func, expr);
+#endif
 }
 
 
